Module: ActiveRecord::Validations::ClassMethods

Defined in:
lib/rails2_ruby2/downcase_patch.rb

Instance Method Summary collapse

Instance Method Details

#validates_uniqueness_of(*attr_names) ⇒ Object



16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
# File 'lib/rails2_ruby2/downcase_patch.rb', line 16

def validates_uniqueness_of(*attr_names)
  configuration = { :case_sensitive => true }
  configuration.update(attr_names.extract_options!)

  validates_each(attr_names,configuration) do |record, attr_name, value|
    # The check for an existing value should be run from a class that
    # isn't abstract. This means working down from the current class
    # (self), to the first non-abstract class. Since classes don't know
    # their subclasses, we have to build the hierarchy between self and
    # the record's class.
    class_hierarchy = [record.class]
    while class_hierarchy.first != self
      class_hierarchy.insert(0, class_hierarchy.first.superclass)
    end

    # Now we can work our way down the tree to the first non-abstract
    # class (which has a database table to query from).
    finder_class = class_hierarchy.detect { |klass| !klass.abstract_class? }

    column = finder_class.columns_hash[attr_name.to_s]

    if value.nil?
      comparison_operator = "IS ?"
    elsif column.text?
      comparison_operator = "#{connection.case_sensitive_equality_operator} ?"
      value = column.limit ? value.to_s.mb_chars[0, column.limit] : value.to_s
    else
      comparison_operator = "= ?"
    end

    sql_attribute = "#{record.class.quoted_table_name}.#{connection.quote_column_name(attr_name)}"

    if value.nil? || (configuration[:case_sensitive] || !column.text?)
      condition_sql = "#{sql_attribute} #{comparison_operator}"
      condition_params = [value]
    else
      condition_sql = "LOWER(#{sql_attribute}) #{comparison_operator}"
      condition_params = [value._mb_chars_downcase]
    end

    if scope = configuration[:scope]
      Array(scope).map do |scope_item|
        scope_value = record.send(scope_item)
        condition_sql << " AND " << attribute_condition("#{record.class.quoted_table_name}.#{connection.quote_column_name(scope_item)}", scope_value)
        condition_params << scope_value
      end
    end

    unless record.new_record?
      condition_sql << " AND #{record.class.quoted_table_name}.#{record.class.primary_key} <> ?"
      condition_params << record.send(:id)
    end

    finder_class.with_exclusive_scope do
      if finder_class.exists?([condition_sql, *condition_params])
        record.errors.add(attr_name, :taken, :default => configuration[:message], :value => value)
      end
    end
  end
end